@@ -20,37 +20,32 @@
 
 """Stash handling."""
 
-from __future__ import absolute_import
-
 import os
 
-from dulwich.file import GitFile
-from dulwich.index import (
-    commit_tree,
-    iter_fresh_objects,
-)
-from dulwich.reflog import read_reflog
-
+from .file import GitFile
+from .index import commit_tree, iter_fresh_objects
+from .reflog import drop_reflog_entry, read_reflog
 
 DEFAULT_STASH_REF = b"refs/stash"
 
 
-class Stash(object):
+class Stash:
     """A Git stash.
 
     Note that this doesn't currently update the working tree.
     """
 
-    def __init__(self, repo, ref=DEFAULT_STASH_REF):
+    def __init__(self, repo, ref=DEFAULT_STASH_REF) -> None:
         self._ref = ref
         self._repo = repo
 
+    @property
+    def _reflog_path(self):
+        return os.path.join(self._repo.commondir(), "logs", os.fsdecode(self._ref))
+
     def stashes(self):
-        reflog_path = os.path.join(
-            self._repo.commondir(), "logs", os.fsdecode(self._ref)
-        )
         try:
-            with GitFile(reflog_path, "rb") as f:
+            with GitFile(self._reflog_path, "rb") as f:
                 return reversed(list(read_reflog(f)))
         except FileNotFoundError:
             return []
@@ -62,10 +57,17 @@ class Stash(object):
 
     def drop(self, index):
         """Drop entry with specified index."""
-        raise NotImplementedError(self.drop)
+        with open(self._reflog_path, "rb+") as f:
+            drop_reflog_entry(f, index, rewrite=True)
+        if len(self) == 0:
+            os.remove(self._reflog_path)
+            del self._repo.refs[self._ref]
+            return
+        if index == 0:
+            self._repo.refs[self._ref] = self[0].new_sha
 
     def pop(self, index):
-        raise NotImplementedError(self.drop)
+        raise NotImplementedError(self.pop)
 
     def push(self, committer=None, author=None, message=None):
         """Create a new stash.
@@ -89,7 +91,8 @@ class Stash(object):
             tree=index_tree_id,
             message=b"Index stash",
             merge_heads=[self._repo.head()],
-            **commit_kwargs
+            no_verify=True,
+            **commit_kwargs,
         )
 
         # Then, the working tree one.
@@ -113,7 +116,8 @@ class Stash(object):
             tree=stash_tree_id,
             message=message,
             merge_heads=[index_commit_id],
-            **commit_kwargs
+            no_verify=True,
+            **commit_kwargs,
         )
 
         return cid
@@ -121,5 +125,5 @@ class Stash(object):
     def __getitem__(self, index):
         return list(self.stashes())[index]
 
-    def __len__(self):
+    def __len__(self) -> int:
         return len(list(self.stashes()))
